require 'stbaldricks/log'
require 'logger'
require 'stbaldricks/default_logger'
require 'securerandom'

module SBF
  module Client
    class Configuration
      DEFAULT_API_ENDPOINT = 'https://api'.freeze

      @logger = nil

      class << self
        attr_writer :base_uri, :user_token, :general_token, :request_id
        attr_accessor :user_token, :forwarded_for
      end

      def self.base_uri
        @base_uri ||= ENV['API_ENDPOINT'] || DEFAULT_API_ENDPOINT
      end

      def self.general_token
        @general_token ||= ENV['API_KEY'] || raise(SBF::Client::Configuration::Error.new('general_token', 'must be set'))
      end

      def self.request_id
        @request_id ||= SecureRandom.uuid
      end

      def self.logger=(new_logger)
        missing_methods = SBF::Client::LOG::REQUIRED_METHODS.reject { |method| new_logger.respond_to?(method) }
        raise SBF::Client::Configuration::Error.new('logger', "must respond to: #{missing_methods.join(', ')}") unless missing_methods.empty?

        @logger = new_logger
      end

      def self.logger
        @logger ||= SBF::Client::DefaultLogger.instance
      end

      class Error < ::StandardError
        def initialize(setting, message) # :nodoc:
          super "SBF::Client::Configuration.#{setting} #{message}"
        end
      end
    end
  end
end
