require 'stbaldricks/endpoints/lib/entity'

module SBF
  module Client
    class CommunicateEndpoint < SBF::Client::EntityEndpoint
      def categories(_ = nil)
        response = SBF::Client::Api::Request.post_request("#{base_uri}/categories")
        parsed_response_body = JSON.parse(response.body).symbolize!

        handle_parsed_response(parsed_response_body, response)
      end

      def years(category)
        response = SBF::Client::Api::Request.post_request("#{base_uri}/years", category: category)
        parsed_response_body = JSON.parse(response.body).symbolize!

        handle_parsed_response(parsed_response_body, response)
      end

      def groups(category, year)
        response = SBF::Client::Api::Request.post_request("#{base_uri}/groups", category: category, year: year)
        parsed_response_body = JSON.parse(response.body).symbolize!

        handle_parsed_response(parsed_response_body, response)
      end

      def compose(entity, with = {})
        raise SBF::Client::Error, 'Invalid Entity' unless entity.is_a?(SBF::Client::BaseEntity)

        with = normalize_with(with)

        create_data = entity.to_hash
        create_data.store(:with, with)

        response = SBF::Client::Api::Request.post_request("#{base_uri}/compose", create_data)

        error = SBF::Client::ErrorEntity.new(response.body) unless ok?(response)

        SBF::Client::Api::Response.new(http_code: response.code, data: nil, error: error)
      end

      def contacts(category, year, group_type, group_id)
        response = SBF::Client::Api::Request.post_request(
          "#{base_uri}/contacts",
          category: category,
          year: year,
          group_type: group_type,
          group_id: group_id
        )
        parsed_response_body = JSON.parse(response.body).symbolize!
        parsed_response_body.map! { |c| SBF::Client::Contact.new(c) } if ok?(response)

        handle_parsed_response(parsed_response_body, response)
      end

      def handle_parsed_response(data, response)
        unless ok?(response)
          error = SBF::Client::ErrorEntity.new(data)
          data = nil
        end

        SBF::Client::Api::Response.new(http_code: response.code, data: data, error: error)
      end
      private :handle_parsed_response
    end
  end
end
