require 'stbaldricks/endpoints/lib/entity'

module SBF
  module Client
    class DeduplicatorMatchEndpoint < SBF::Client::EntityEndpoint
      def checkout(filter = [], order = {}, limit = 20, offset = 0, with = {})
        filter = filter.to_json unless filter.is_a? String
        order = order.to_json unless order.is_a? String
        with = normalize_with(with)

        response = SBF::Client::Api::Request.post_request(
          "#{base_uri}/checkout",
          filter: filter, order: order, limit: limit, offset: offset, with: with
        )
        parsed_response_body = JSON.parse(response.body).symbolize!

        if ok?(response)
          parsed_response_body[:results].map! { |entity_data| target_class.new(entity_data, true) }
          SBF::Client::EntityCollection.new(parsed_response_body[:results], parsed_response_body[:total_count])
        else
          parsed_response_body = JSON.parse(response.body).symbolize!
          error = SBF::Client::ErrorEntity.new(parsed_response_body)
          SBF::Client::Api::Response.new(http_code: response.code, data: parsed_response_body[:data], error: error)
          collection = SBF::Client::EntityCollection.new
          collection.add_errors(error)
          collection.errors_http_code = response.code
          collection
        end
      end

      def list_checkouts
        response = SBF::Client::Api::Request.post_request(
          "#{base_uri}/list_checkouts"
        )
        parsed_response_body = JSON.parse(response.body).symbolize!

        if ok?(response)
          data = parsed_response_body
        else
          error = SBF::Client::ErrorEntity.new(parsed_response_body)
        end

        SBF::Client::Api::Response.new(http_code: response.code, data: data, error: error)
      end

      def release_checkout(checkout_id)
        response = SBF::Client::Api::Request.post_request(
          "#{base_uri}/release_checkout", checkout_id: checkout_id
        )
        parsed_response_body = JSON.parse(response.body).symbolize!

        if ok?(response)
          data = parsed_response_body
        else
          error = SBF::Client::ErrorEntity.new(parsed_response_body)
        end

        SBF::Client::Api::Response.new(http_code: response.code, data: data, error: error)
      end
    end
  end
end
