require 'stbaldricks/entities/event'
require 'stbaldricks/entities/team'
require 'stbaldricks/entities/participant'
require 'stbaldricks/entities/fundraiser'
require 'stbaldricks/entities/lib/base'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/default_cacheable'
require 'stbaldricks/entities/challenger/model_type'
require 'stbaldricks/entities/concerns/type_concern'

module SBF
  module Client
    class Challenger < SBF::Client::TopLevelEntity
      include Entities::DefaultCacheable
      actions DEFAULT_CRUD_ACTIONS
      blacklist_action :delete
      blacklist_action :create

      module Type
        CHALLENGER = 'challenger'.freeze
        CHALLENGEE = 'challengee'.freeze
      end

      class Totals < SBF::Client::BaseEntity
        attr_reader :amount_raised
      end

      class FullParticipant < SBF::Client::FullParticipant
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Challenger::ModelType::PARTICIPANT
        end
      end

      class PartialParticipant < SBF::Client::PartialParticipant
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Challenger::ModelType::PARTICIPANT
        end
      end

      class FullFundraiser < SBF::Client::FullFundraiser
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Challenger::ModelType::FUNDRAISER
        end
      end

      class PartialFundraiser < SBF::Client::PartialFundraiser
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Challenger::ModelType::FUNDRAISER
        end
      end

      class FullTeam < SBF::Client::FullTeam
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Challenger::ModelType::TEAM
        end
      end

      class PartialTeam < SBF::Client::PartialTeam
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Challenger::ModelType::TEAM
        end
      end

      class FullEvent < SBF::Client::FullEvent
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Challenger::ModelType::EVENT
        end
      end

      class PartialEvent < SBF::Client::PartialEvent
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Challenger::ModelType::EVENT
        end
      end

      attr_accessor :id
      attr_accessor :challenge_id
      attr_accessor :type
      attr_accessor :is_accepted
      entity_attr_reader :totals, 'SBF::Client::Challenger::Totals', nil, true

      multitype_attr_accessor(
        :entity,
        [
          [
            ->(v) { v[:type] == SBF::Client::Challenger::ModelType::PARTICIPANT },
            'SBF::Client::Challenger::FullParticipant',
            'SBF::Client::Challenger::PartialParticipant'
          ],
          [
            ->(v) { v[:type] == SBF::Client::Challenger::ModelType::FUNDRAISER },
            'SBF::Client::Challenger::FullFundraiser',
            'SBF::Client::Challenger::PartialFundraiser'
          ],
          [
            ->(v) { v[:type] == SBF::Client::Challenger::ModelType::TEAM },
            'SBF::Client::Challenger::FullTeam',
            'SBF::Client::Challenger::PartialTeam'
          ],
          [
            ->(v) { v[:type] == SBF::Client::Challenger::ModelType::EVENT },
            'SBF::Client::Challenger::FullEvent',
            'SBF::Client::Challenger::PartialEvent'
          ]
        ],
        true
      )
    end
  end
end
