require 'singleton'
require 'stbaldricks/endpoints/config'
require 'stbaldricks/entities/lib/top_level'

module SBF
  module Client
    class Config < SBF::Client::TopLevelEntity
      include Singleton
      endpoint SBF::Client::ConfigEndpoint

      action :get
      instance_action :reload

      attr_reader :default_year
      attr_reader :open_registration_years
      attr_reader :website_url
      attr_reader :years_for_squire
      attr_reader :years_for_knight
      attr_reader :years_for_knight_commander
      attr_reader :years_for_crusader
      attr_reader :years_for_baron
      # NOTE: If you add another attribute, be sure to mock it out in `configure :test` of app.rb

      # Attempts to call the method on the cached configuration object
      # rubocop:disable Style/MissingRespondToMissing
      def self.method_missing(method_name, *arguments, &block)
        # The first time a missing method is called, call reload to make sure the instance has been populated
        unless @_loaded
          @_loaded = true
          reload
        end

        # If the method being called exists on the instance, call it. Otherwise forward the request on to 'super'.
        return instance.send(method_name) if instance.respond_to?(method_name)

        super(method_name, *arguments, &block)
      end
      # rubocop:enable Style/MissingRespondToMissing
    end
  end
end
