require 'stbaldricks/entities/fund'
require 'stbaldricks/entities/campaign'
require 'stbaldricks/entities/event'
require 'stbaldricks/entities/team'
require 'stbaldricks/entities/participant'
require 'stbaldricks/entities/fundraiser'
require 'stbaldricks/entities/lib/address'
require 'stbaldricks/entities/lib/base'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/name'
require 'stbaldricks/entities/concerns/model_type_concern'

module SBF
  module Client
    class EventApplication < SBF::Client::TopLevelEntity
      actions DEFAULT_CRUD_ACTIONS
      blacklist_action :delete

      class FullParticipant < SBF::Client::FullParticipant
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::EventApplication::ModelType::PARTICIPANT
        end
      end

      class PartialParticipant < SBF::Client::PartialParticipant
        include ModelTypeConcern
        attr_reader :model_type
        attr_accessor :year

        def model_type
          SBF::Client::EventApplication::ModelType::PARTICIPANT
        end
      end

      class FullFundraiser < SBF::Client::FullFundraiser
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::EventApplication::ModelType::FUNDRAISER
        end
      end

      class PartialFundraiser < SBF::Client::PartialFundraiser
        include ModelTypeConcern
        attr_reader :model_type
        attr_accessor :year

        def model_type
          SBF::Client::EventApplication::ModelType::FUNDRAISER
        end
      end

      class FullTeam < SBF::Client::FullTeam
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::EventApplication::ModelType::TEAM
        end
      end

      class PartialTeam < SBF::Client::PartialTeam
        include ModelTypeConcern
        attr_reader :model_type
        attr_accessor :year

        def model_type
          SBF::Client::EventApplication::ModelType::TEAM
        end
      end

      class FullEvent < SBF::Client::FullEvent
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::EventApplication::ModelType::EVENT
        end
      end

      class PartialEvent < SBF::Client::PartialEvent
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::EventApplication::ModelType::EVENT
        end
      end

      class Requestor < SBF::Client::BaseEntity
        attr_accessor :name
        entity_attr_accessor :name_pieces, 'SBF::Client::NamePieces'
        entity_attr_accessor :email_address, 'SBF::Client::EmailAddress'
        entity_attr_accessor :phone, 'SBF::Client::Phone'
        entity_attr_accessor :address, 'SBF::Client::Address'
      end

      attr_accessor :id
      attr_accessor :requested_event_title
      attr_accessor :status
      attr_accessor :is_archived
      attr_accessor :no_conversion_reason
      attr_accessor :promotional_code
      attr_accessor :created_at, :modified_at
      entity_attr_accessor :requestor, 'SBF::Client::EventApplication::Requestor'
      entity_attr_accessor :qualifier, 'SBF::Client::FullPerson', 'SBF::Client::PartialPerson', true
      entity_attr_accessor :fund, 'SBF::Client::FullFund', 'SBF::Client::PartialFund', true
      entity_attr_accessor :campaign, 'SBF::Client::FullCampaign', 'SBF::Client::PartialCampaign', true

      multitype_attr_accessor(
        :model,
        [
          [
            ->(v) { v[:model_type] == SBF::Client::EventApplication::ModelType::PARTICIPANT },
            'SBF::Client::EventApplication::FullParticipant',
            'SBF::Client::EventApplication::PartialParticipant'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::EventApplication::ModelType::FUNDRAISER },
            'SBF::Client::EventApplication::FullFundraiser',
            'SBF::Client::EventApplication::PartialFundraiser'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::EventApplication::ModelType::TEAM },
            'SBF::Client::EventApplication::FullTeam',
            'SBF::Client::EventApplication::PartialTeam'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::EventApplication::ModelType::EVENT },
            'SBF::Client::EventApplication::FullEvent',
            'SBF::Client::EventApplication::PartialEvent'
          ]
        ],
        optional: true
      )
    end
  end
end
