require 'stbaldricks/entities/fund'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/fundraising_page'
require 'stbaldricks/endpoints/fund'
require 'stbaldricks/entities/lib/default_cacheable'
require 'stbaldricks/entities/concerns/donation_recipient_concern'

module SBF
  module Client
    class Fund < SBF::Client::TopLevelEntity
      include DonationRecipientConcern, Entities::DefaultCacheable
      endpoint SBF::Client::FundEndpoint
      action :allowed_entities
      action :get
      action :find
      action :find_first
      action :update
      action :save

      disallow_instantiation

      class Photos < SBF::Client::BaseEntity
        attr_reader :avatar, :default
      end

      class YearlyTotals < SBF::Client::BaseEntity
        attr_reader :year
        attr_reader :fundraisers
        attr_reader :participants
        attr_reader :shavees
        attr_reader :non_shavees
        attr_reader :teams
        attr_reader :events
        attr_reader :amount_raised

        def to_hash
          {}.tap do |hsh|
            self.class.defined_attributes.each { |name| hsh[name] = send(name.to_sym) unless name.to_sym == :year }
          end
        end
      end

      class Totals < SBF::Client::BaseEntity
        attr_reader :fundraisers
        attr_reader :participants
        attr_reader :shavees
        attr_reader :non_shavees
        attr_reader :teams
        attr_reader :events
        attr_reader :amount_raised
      end

      class Photos < SBF::Client::BaseEntity
        attr_reader :avatar, :default
      end
    end

    class PartialFund < Fund
      attr_accessor :id
    end

    class FullFund < Fund
      attr_accessor :id
      attr_reader :display_name
      attr_accessor :name
      attr_accessor :disabled
      attr_accessor :description
      entity_attr_accessor :photos, 'SBF::Client::Fund::Photos'
      entity_attr_reader :totals, 'SBF::Client::Fund::Totals'
      entity_collection_attr_accessor :totals_by_year, 'SBF::Client::Fund::Total'
      entity_attr_accessor :fundraising_page, 'SBF::Client::FundraisingPage', nil, true
      entity_attr_accessor :photos, 'SBF::Client::Fund::Photos'
      attr_accessor :kid_name
      attr_accessor :offline_donation_uuid, :offline_donation_form

      def totals_by_year=(value)
        if value.nil? || value.empty?
          @totals_by_year = []
          return
        end

        totals_by_year_will_change!
        @totals_by_year = value.map do |k, v|
          SBF::Client::Fund::YearlyTotals.new(v.merge(year: k.to_s.to_i))
        end
      end

      def to_hash
        hsh = super.to_hash
        return hsh if @totals_by_year.nil?

        hsh[:totals_by_year] = {}
        @totals_by_year.each do |year_total|
          hsh[:totals_by_year][year_total.year.to_s.to_sym] = year_total.to_hash
        end

        hsh
      end
    end
  end
end
