require 'stbaldricks/entities/disease'
require 'stbaldricks/entities/institution'
require 'stbaldricks/entities/lib/name'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/base'
require 'stbaldricks/entities/lib/default_cacheable'

module SBF
  module Client
    class Grant < SBF::Client::TopLevelEntity
      include Entities::DefaultCacheable

      actions DEFAULT_CRUD_ACTIONS
      blacklist_action :create
      blacklist_action :update
      blacklist_action :delete

      disallow_instantiation

      class PartialResearcher < SBF::Client::BaseEntity
        attr_accessor :id
      end

      class FullResearcher < SBF::Client::BaseEntity
        attr_accessor :id
        attr_reader :display_name
        attr_accessor :description
        attr_accessor :photo_url
        entity_attr_accessor :name_pieces, 'SBF::Client::NamePieces'
        attr_reader :created_at
        attr_reader :modified_at
      end

      class PartialFundingType < SBF::Client::BaseEntity
        attr_accessor :id
      end

      class FullFundingType < SBF::Client::BaseEntity
        attr_accessor :id
        attr_accessor :name
        attr_accessor :description
        attr_reader :created_at
        attr_reader :modified_at
      end

      class PartialInstitution < SBF::Client::PartialInstitution; end

      class FullInstitution < SBF::Client::FullInstitution; end

      class Disease < SBF::Client::Disease; end
    end

    class PartialGrant < SBF::Client::Grant
      attr_accessor :id
    end

    class FullGrant < SBF::Client::Grant
      attr_accessor :id
      attr_accessor :name
      attr_accessor :funding_begin_date
      attr_accessor :funding_end_date
      attr_accessor :web_message
      attr_accessor :article_url
      attr_accessor :amount
      entity_attr_accessor :researcher, 'SBF::Client::Grant::FullResearcher', 'SBF::Client::Grant::PartialResearcher', true
      entity_attr_accessor :institution, 'SBF::Client::Grant::FullInstitution', 'SBF::Client::Grant::PartialInstitution', true
      entity_attr_accessor :funding_type, 'SBF::Client::Grant::FullFundingType', 'SBF::Client::Grant::PartialFundingType', true
      entity_collection_attr_accessor :diseases, 'SBF::Client::Grant::Disease', nil, true
      attr_reader :photo_url
      attr_reader :created_at
      attr_reader :modified_at
    end
  end
end
