require 'stbaldricks/endpoints/kid'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/address'
require 'stbaldricks/entities/lib/name'
require 'stbaldricks/entities/lib/fundraising_page'
require 'stbaldricks/entities/institution'
require 'stbaldricks/entities/person'
require 'stbaldricks/entities/lib/base'
require 'stbaldricks/entities/lib/default_cacheable'

module SBF
  module Client
    class Diagnosis < SBF::Client::BaseEntity
      attr_accessor :date
      attr_accessor :cancer_type
      attr_accessor :other

      module CancerType
        ACUTE_LYMPHOBLASTIC_LEUKEMIA = 'acute_lymphoblastic_leukemia'.freeze
        ACUTE_MYELOID_LEUKEMIA = 'acute_myeloid_leukemia'.freeze
        BRAIN_OR_SPINAL_CORD_TUMOR = 'brain_or_spinal_cord_tumor'.freeze
        KIDNEY_SARCOMA = 'kidney_sarcoma'.freeze
        EWING_SARCOMA = 'ewing_sarcoma'.freeze
        GERM_CELL_TUMOR = 'germ_cell_tumor'.freeze
        HEPATOBLASTOMA = 'hepatoblastoma'.freeze
        HODGKIN_LYMPHOMA = 'hodgkin_lymphoma'.freeze
        LANGERHANS_CELL_HISTIOCYTOSIS = 'langerhans_cell_histiocytosis'.freeze
        LYMPHOMA = 'lymphoma'.freeze
        NEUROBLASTOMA = 'neuroblastoma'.freeze
        NON_HODGKIN_LYMPHOMA = 'non_hodgkin_lymphoma'.freeze
        OSTEOSARCOMA = 'osteosarcoma'.freeze
        RETINOBLASTOMA = 'retinoblastoma'.freeze
        RHABDOMYOSARCOMA = 'rhabdomyosarcoma'.freeze
        WILMS_OR_OTHER_KIDNEY_TUMOR = 'wilms_or_other_kidney_tumor'.freeze
        ADULT_WITH_CHILDHOOD_CANCER = 'adult_with_childhood_cancer'.freeze
        OTHER_CHILDHOOD_CANCER = 'other_childhood_cancer'.freeze
      end
    end

    class Guardian < SBF::Client::BaseEntity
      module RelationshipType
        SELF_ADULT = 'adult'.freeze
        SELF_MINOR = 'minor'.freeze
        MOTHER = 'mother'.freeze
        FATHER = 'father'.freeze
        SISTER = 'sister'.freeze
        BROTHER = 'brother'.freeze
        AUNT = 'aunt'.freeze
        UNCLE = 'uncle'.freeze
        GRANDMOTHER = 'grandmother'.freeze
        GRANDFATHER = 'grandfather'.freeze
        STEP_MOTHER = 'stepmother'.freeze
        STEP_FATHER = 'stepfather'.freeze
        FRIEND = 'friend'.freeze
        COUSIN = 'cousin'.freeze
      end

      attr_accessor :name
      attr_accessor :relationship
      entity_attr_accessor :name_pieces, 'SBF::Client::NamePieces'
      entity_attr_accessor :email_address, 'SBF::Client::EmailAddress'
      entity_attr_accessor :phone, 'SBF::Client::Phone'
    end

    class TreatmentStatus < SBF::Client::BaseEntity
      attr_accessor :state
      attr_accessor :other

      module SurvivorState
        IN_TREATMENT = 'in_treatment'.freeze
        IN_MAINTENANCE = 'in_maintenance'.freeze
        IN_REMISSION = 'in_remission'.freeze
        NED = 'no_evidence_of_disease'.freeze
        CANCER_FREE = 'cancer_free'.freeze
        OTHER_LIVING = 'other_living'.freeze
      end

      module DeceasedState
        ANGEL = 'angel'.freeze
        DECEASED = 'deceased'.freeze
        PASSED_AWAY = 'passed_away'.freeze
        OTHER_DECEASED = 'other_deceased'.freeze
      end
    end

    class KidURL < SBF::Client::BaseEntity
      attr_accessor :url
    end

    class Photos < SBF::Client::BaseEntity
      attr_reader :avatar, :default
    end

    class Kid < SBF::Client::TopLevelEntity
      include Entities::DefaultCacheable
      endpoint SBF::Client::KidEndpoint
      actions DEFAULT_CRUD_ACTIONS
      action :find_random
      blacklist_action :create
      blacklist_action :delete

      disallow_instantiation

      module Status
        ACTIVE = 'active'.freeze
        PENDING = 'pending'.freeze
        SUSPENDED = 'suspended'.freeze
      end

      class Relationship < SBF::Client::BaseEntity
        module Type
          SELF_ADULT = 'adult'.freeze
          SELF_MINOR = 'minor'.freeze
          MOTHER = 'mother'.freeze
          FATHER = 'father'.freeze
          SISTER = 'sister'.freeze
          BROTHER = 'brother'.freeze
          AUNT = 'aunt'.freeze
          UNCLE = 'uncle'.freeze
          GRANDMOTHER = 'grandmother'.freeze
          GRANDFATHER = 'grandfather'.freeze
          STEP_MOTHER = 'stepmother'.freeze
          STEP_FATHER = 'stepfather'.freeze
          FRIEND = 'friend'.freeze
          COUSIN = 'cousin'.freeze
        end

        module GuardianType
          LEGAL_GUARDIAN = 'legal_guardian'.freeze
          NONCUSTODIAL_GUARDIAN = 'non_custodial_guardian'.freeze
          OTHER = 'other'.freeze
        end

        class Permissions < SBF::Client::BaseEntity
          attr_accessor :is_editor
        end

        attr_accessor :kid_id
        entity_attr_accessor :person, 'SBF::Client::FullPerson', 'SBF::Client::PartialPerson'
        attr_accessor :type
        attr_accessor :guardian_type
        attr_accessor :is_submitter
        entity_attr_accessor :permissions, 'SBF::Client::Kid::Relationship::Permissions'
      end

      class CustomInstitution < SBF::Client::FullInstitution; end
    end

    class PartialKid < SBF::Client::Kid
      attr_accessor :id
    end

    class FullKid < SBF::Client::Kid
      extend Forwardable
      def_delegator :@photos, :avatar, :avatar_photo_url

      attr_accessor :id
      attr_accessor :status
      attr_accessor :gender
      attr_accessor :birthday
      attr_accessor :ethnicity
      attr_accessor :date_of_passing
      attr_accessor :display_name
      attr_accessor :display_name_override
      attr_accessor :name
      entity_attr_accessor :name_pieces, 'SBF::Client::NamePieces'
      entity_attr_accessor :address, 'SBF::Client::Address'
      entity_attr_accessor :diagnosis, 'SBF::Client::Diagnosis'
      entity_attr_accessor :treatment_status, 'SBF::Client::TreatmentStatus'
      attr_accessor :web_message
      entity_attr_accessor :photos, 'SBF::Client::Photos'
      entity_attr_accessor :web_page, 'SBF::Client::WebPage'
      entity_attr_accessor :guardian, 'SBF::Client::Guardian'
      entity_collection_attr_accessor :relationships, 'SBF::Client::Kid::Relationship'
      entity_collection_attr_accessor :urls, 'SBF::Client::KidURL'
      entity_collection_attr_accessor :custom_institutions, 'SBF::Client::Kid::CustomInstitution', nil, true
      entity_attr_accessor :third_party_media, 'SBF::Client::ThirdPartyMedia', nil, true

      attr_accessor :created_at
      attr_accessor :modified_at
      attr_accessor :modified_by

      def fundraising_page_url
        web_page.url
      end

      def active?
        status == SBF::Client::Kid::Status::ACTIVE
      end

      def suspended?
        status == SBF::Client::Kid::Status::SUSPENDED
      end
    end
  end
end
