require 'stbaldricks/endpoints/kid_honor'
require 'stbaldricks/entities/kid'
require 'stbaldricks/entities/event'
require 'stbaldricks/entities/team'
require 'stbaldricks/entities/participant'
require 'stbaldricks/entities/fundraiser'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/concerns/model_type_concern'

module SBF
  module Client
    class KidHonor < SBF::Client::TopLevelEntity
      endpoint SBF::Client::KidHonorEndpoint
      actions DEFAULT_CRUD_ACTIONS
      blacklist_action :get
      blacklist_action :update

      attr_accessor :honored_on
      entity_attr_accessor :kid, 'SBF::Client::KidHonor::FullKid', 'SBF::Client::KidHonor::PartialKid'
      multitype_attr_accessor(
        :honorer,
        [
          [
            ->(v) { v[:model_type] == SBF::Client::KidHonor::HonorerType::PARTICIPANT },
            'SBF::Client::KidHonor::FullParticipant',
            'SBF::Client::KidHonor::PartialParticipant'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::KidHonor::HonorerType::FUNDRAISER },
            'SBF::Client::KidHonor::FullFundraiser',
            'SBF::Client::KidHonor::PartialFundraiser'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::KidHonor::HonorerType::TEAM },
            'SBF::Client::KidHonor::FullTeam',
            'SBF::Client::KidHonor::PartialTeam'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::KidHonor::HonorerType::EVENT },
            'SBF::Client::KidHonor::FullEvent',
            'SBF::Client::KidHonor::PartialEvent'
          ]
        ]
      )

      module HonorerType
        EVENT = 'event'.freeze
        TEAM = 'team'.freeze
        PARTICIPANT = 'participant'.freeze
        FUNDRAISER = 'fundraiser'.freeze
      end

      class PartialKid < SBF::Client::PartialKid
      end

      class FullKid < SBF::Client::FullKid
      end

      class FullParticipant < SBF::Client::FullParticipant
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::KidHonor::HonorerType::PARTICIPANT
        end
      end

      class PartialParticipant < SBF::Client::PartialParticipant
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::KidHonor::HonorerType::PARTICIPANT
        end
      end

      class FullFundraiser < SBF::Client::FullFundraiser
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::KidHonor::HonorerType::FUNDRAISER
        end
      end

      class PartialFundraiser < SBF::Client::PartialFundraiser
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::KidHonor::HonorerType::FUNDRAISER
        end
      end

      class FullTeam < SBF::Client::FullTeam
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::KidHonor::HonorerType::TEAM
        end
      end

      class PartialTeam < SBF::Client::PartialTeam
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::KidHonor::HonorerType::TEAM
        end
      end

      class FullEvent < SBF::Client::FullEvent
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::KidHonor::HonorerType::EVENT
        end
      end

      class PartialEvent < SBF::Client::PartialEvent
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::KidHonor::HonorerType::EVENT
        end
      end
    end
  end
end
