require 'stbaldricks/errors'
require 'active_model'

module SBF
  module Client
    class ErrorEntity
      BAD_INPUT = 5007

      attr_reader :code, :type, :details, :fields

      def initialize(data = {})
        super()

        # In case null was explicitly passed in
        data ||= {}

        # Set all attributes
        @code = data[:code].freeze
        @type = data[:type].freeze
        @details = data[:details].freeze
        @fields = Set.new(data[:errors].empty? ? [] : data[:errors]).freeze
      end

      def empty?
        [@code, @type, @details].all?(&:nil?) && @fields.empty?
      end

      def to_hash
        {code: @code, type: @type, details: @details, fields: @fields.to_a}
      end

      def to_json(*a)
        to_hash.to_json(*a)
      end

      def to_s
        error_message = "#{@code} - [#{@type}] #{@details}"
        error_message << " details: #{@fields.to_a.join(', ')}" unless @fields.empty?
        error_message
      end
    end

    class ApiErrorEntity < ErrorEntity
      attr_reader :http_code

      def initialize(data)
        super(data)
        @http_code = data[:http_code].freeze
      end
    end

    module Entity
      class Errors < ActiveModel::Errors
        attr_reader :http_code, :code, :type

        def initialize(obj, http_code = nil)
          super(obj)
          @http_code = http_code
        end
      end
    end
  end
end
