require 'stbaldricks/entities/lib/base'
require 'stbaldricks/entities/concerns/type_concern'

module SBF
  module Client
    class Payment
      module Type
        APPLEPAY = 'applepay'.freeze
        CASH = 'cash'.freeze
        CHECK = 'check'.freeze
        CREDIT_CARD = 'credit_card'.freeze
        FACEBOOK = 'facebook'.freeze
        GOOGLEPAY = 'google_pay'.freeze
        MONEY_ORDER = 'money_order'.freeze
        NONCE = 'nonce'.freeze
        PAYPAL = 'paypal'.freeze
        STRIPE = 'stripe'.freeze
        UNKNOWN = 'unknown'.freeze
        WIRE_TRANSFER = 'wire_transfer'.freeze
      end

      module Gateway
        NONE = 'none'.freeze
        UNKNOWN = 'unknown'.freeze
        BRAINTREE = 'braintree'.freeze
        PAYMENTSPRING = 'paymentspring'.freeze
      end

      class Details < SBF::Client::BaseEntity
        include TypeConcern
        attr_reader :type

        def type
          SBF::Client::Payment::Type::UNKNOWN
        end
      end

      class SubscriptionCreditCardDetails < SBF::Client::Payment::Details
        attr_accessor :gateway, :cardholder_name, :card_type, :expiration_date, :card_number

        def type
          SBF::Client::Payment::Type::CREDIT_CARD
        end
      end

      class CreditCardDetails < SBF::Client::Payment::Details
        attr_accessor :gateway, :authorization_id, :cardholder_name, :card_type, :expiration_date, :card_number

        def type
          SBF::Client::Payment::Type::CREDIT_CARD
        end
      end

      class CheckDetails < SBF::Client::Payment::Details
        attr_accessor :gateway, :number

        def type
          SBF::Client::Payment::Type::CHECK
        end
      end

      class MoneyOrderDetails < SBF::Client::Payment::Details
        attr_accessor :number

        def type
          SBF::Client::Payment::Type::MONEY_ORDER
        end
      end

      class WireTransferDetails < SBF::Client::Payment::Details
        attr_accessor :number

        def type
          SBF::Client::Payment::Type::WIRE_TRANSFER
        end
      end

      class CashDetails < SBF::Client::Payment::Details
        def type
          SBF::Client::Payment::Type::CASH
        end
      end

      class UnknownDetails < SBF::Client::Payment::Details
        def type
          SBF::Client::Payment::Type::UNKNOWN
        end
      end

      class NonceDetails < SBF::Client::Payment::Details
        attr_accessor :gateway, :nonce, :device_data

        def type
          SBF::Client::Payment::Type::NONCE
        end
      end

      class PaypalDetails < SBF::Client::Payment::Details
        attr_accessor :gateway

        def type
          SBF::Client::Payment::Type::PAYPAL
        end
      end

      class FacebookDetails < SBF::Client::Payment::Details
        attr_accessor :number, :authorization_id

        def type
          SBF::Client::Payment::Type::FACEBOOK
        end
      end

      class ApplepayDetails < SBF::Client::Payment::Details
        attr_accessor :gateway

        def type
          SBF::Client::Payment::Type::APPLEPAY
        end
      end

      class StripeDetails < SBF::Client::Payment::Details
        def type
          SBF::Client::Payment::Type::STRIPE
        end
      end

      class GooglePayDetails < SBF::Client::Payment::Details
        attr_accessor :gateway

        def type
          SBF::Client::Payment::Type::GOOGLEPAY
        end
      end
    end
  end
end
