require 'stbaldricks/entities/lib/base'
require 'stbaldricks/endpoints/lib/entity'
require 'stbaldricks/entities/lib/cacheable'

module SBF
  module Client
    class TopLevelEntity < SBF::Client::BaseEntity
      include Entities::Cacheable

      DEFAULT_ENDPOINT = SBF::Client::EntityEndpoint
      DEFAULT_INSTANCE_ACTIONS = %i(save create update delete).freeze
      DEFAULT_CLASS_ACTIONS = %i(get find find_first aggregate update delete).freeze
      DEFAULT_CRUD_ACTIONS = DEFAULT_INSTANCE_ACTIONS + DEFAULT_CLASS_ACTIONS

      class << self
        DEFAULT_CLASS_ACTIONS.each do |action|
          define_method(action) do |*_|
            raise NoMethodError.new("#{action} is not implemented", action)
          end
        end

        def endpoint(mod = nil, options = {})
          return define_endpoint(mod, options) if mod

          _endpoint
        end

        def _endpoint
          DEFAULT_ENDPOINT.new(self)
        end

        def define_endpoint(mod, options)
          singleton_class.class_eval do
            define_method(:_endpoint) do
              @_endpoint ||= mod.new(options.fetch(:target_class, self))
            end
          end
        end

        def actions(actions)
          actions.each { |m| action(m) }
        end

        def instance_action(name)
          define_method(name) do |*args|
            result = endpoint.send(name, self, *args)
            result unless result.is_a?(SBF::Client::TopLevelEntity)
            %i(save create).include?(name) && result.errors? ? false : result
          end
        end

        def class_action(name)
          singleton_class.class_eval do
            define_method(name) do |*args|
              endpoint.send(name, *args)
            end
          end
        end

        def action(name)
          instance_action(name) if %i(save create).include?(name)

          if name == :update
            define_method(name) do |*args|
              result = endpoint.update(id, self, *args)
              result unless result.is_a?(SBF::Client::TopLevelEntity)
              result.errors? ? false : result
            end
          end

          if name == :delete
            define_method(:delete) do |*_args|
              response = endpoint.delete(id)
              if response.error?
                add_errors(response.error)
                errors.instance_variable_set(:@http_code, response.http_code)
                false
              else
                @destroyed = true
                freeze
              end
            end
          end

          class_action(name)
        end

        def blacklist_action(name)
          define_method(name) do |*_|
            raise NoMethodError.new("#{name} is blacklisted", name)
          end

          singleton_class.class_eval do
            define_method(name) do |*_|
              raise NoMethodError.new("#{name} is blacklisted", name)
            end
          end
        end
      end

      DEFAULT_INSTANCE_ACTIONS.each do |action|
        define_method(action) do |*_|
          raise NoMethodError.new("#{action} is not implemented", action)
        end
      end

      def endpoint
        self.class.endpoint
      end
    end
  end
end
