require 'stbaldricks/endpoints/matching_gift_company'
require 'stbaldricks/entities/lib/top_level'

module SBF
  module Client
    module MatchingGift
      class Company < SBF::Client::TopLevelEntity
        endpoint SBF::Client::MatchingGiftCompanyEndpoint
        action :get
        action :find

        disallow_instantiation

        class Contact < SBF::Client::BaseEntity
          attr_reader :title
          attr_reader :name
          entity_attr_reader :name_pieces, 'SBF::Client::NamePieces'
          entity_attr_reader :address, 'SBF::Client::Address'
          entity_attr_reader :phone, 'SBF::Client::Phone'
          entity_attr_reader :email, 'SBF::Client::EmailAddress'
          attr_reader :websites
        end

        class GiftRatio < SBF::Client::BaseEntity
          attr_reader :match_ratio
          attr_reader :minimum_matched
          attr_reader :maximum_matched
          attr_reader :total_matched_per_employee
          attr_reader :fiscal_year
          attr_reader :deadline
        end

        class EmployeeEligibility < SBF::Client::BaseEntity
          attr_reader :full_time
          attr_reader :part_time
          attr_reader :retired
          attr_reader :spouse
          attr_reader :board
        end

        class Procedures < SBF::Client::BaseEntity
          attr_reader :officer_certification
          attr_reader :fax
          attr_reader :website
          attr_reader :phone
          attr_reader :presenting_evidence
          attr_reader :sending_check
          attr_reader :sending_letter
        end

        class Requirements < SBF::Client::BaseEntity
          attr_reader :gift_receipt
          attr_reader :ack_letter
        end

        class Restrictions < SBF::Client::BaseEntity
          attr_reader :follow_donor_designation
          attr_reader :match_unrestricted
        end
      end

      class PartialCompany < Company
        attr_reader :name
        attr_reader :id
      end

      class FullCompany < Company
        attr_reader :id
        attr_reader :name
        attr_reader :matching_gift_forms
        attr_reader :special_comments
        attr_reader :last_updated

        entity_attr_reader :contact_info, 'SBF::Client::MatchingGift::Company::Contact', nil, true
        entity_attr_reader :match_info, 'SBF::Client::MatchingGift::Company::GiftRatio', nil, true
        entity_attr_reader :employee_eligibility, 'SBF::Client::MatchingGift::Company::EmployeeEligibility', nil, true
        entity_attr_reader :procedures, 'SBF::Client::MatchingGift::Company::Procedures', nil, true
        entity_attr_reader :requirements, 'SBF::Client::MatchingGift::Company::Requirements', nil, true
        entity_attr_reader :restrictions, 'SBF::Client::MatchingGift::Company::Restrictions', nil, true
      end
    end
  end
end
