require 'stbaldricks/entities/lib/address'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/third_party_media'
require 'stbaldricks/entities/lib/default_cacheable'
require 'stbaldricks/entities/concerns/donation_recipient_concern'

module SBF
  module Client
    class Memorial < SBF::Client::TopLevelEntity
      include DonationRecipientConcern, Entities::DefaultCacheable
      actions DEFAULT_CRUD_ACTIONS
      blacklist_action :save
      blacklist_action :create
      blacklist_action :delete

      disallow_instantiation

      module Status
        ACTIVE = 'active'.freeze
        PENDING = 'pending'.freeze
      end

      class Tribute < SBF::Client::BaseEntity
        attr_accessor :birthday
        attr_accessor :date_of_passing
        attr_accessor :name
      end

      class Totals < SBF::Client::BaseEntity
        attr_reader :amount_raised
        attr_reader :donations
      end

      class Photos < SBF::Client::BaseEntity
        attr_reader :avatar, :default, :cover, :cover_original
      end
    end

    class PartialMemorial < Memorial
      attr_accessor :id
    end

    class FullMemorial < Memorial
      attr_accessor :id
      attr_accessor :status
      attr_accessor :website_message
      attr_reader :display_name
      entity_attr_accessor :address, 'SBF::Client::Address'
      entity_attr_accessor :tribute, 'SBF::Client::Memorial::Tribute'
      entity_attr_reader :totals, 'SBF::Client::Memorial::Totals'
      entity_collection_attr_accessor :donations, 'SBF::Client::FullDonation', 'SBF::Client::PartialDonation'
      attr_reader :created_at, :modified_at
      attr_accessor :expires_at
      entity_attr_accessor :photos, 'SBF::Client::Memorial::Photos'
      entity_attr_accessor :third_party_media, 'SBF::Client::ThirdPartyMedia', nil, true
      attr_accessor :offline_donation_uuid, :offline_donation_form
      entity_attr_accessor :fundraising_page, 'SBF::Client::FundraisingPage', nil, true

      def active?
        status == SBF::Client::Memorial::Status::ACTIVE
      end
    end
  end
end
