require 'stbaldricks/entities/lib/profile_address'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/profile_email_address'
require 'stbaldricks/entities/lib/name'
require 'stbaldricks/entities/lib/phone'
require 'stbaldricks/entities/lib/opt_out_settings'
require 'stbaldricks/entities/lib/alternate_shipping_address'
require 'stbaldricks/entities/concerns/type_concern'

module SBF
  module Client
    class Organization < SBF::Client::TopLevelEntity
      include TypeConcern
      actions DEFAULT_CRUD_ACTIONS

      disallow_instantiation

      module Status
        PENDING = 'pending'.freeze
        ACTIVE = 'active'.freeze
        DECEASED = 'deceased'.freeze
      end

      module Type
        BUSINESS = 'business'.freeze
        ORGANIZATION_FOUNDATION = 'foundation'.freeze
      end

      module Gender
        NOT_SELECTED = 'not_selected'.freeze
      end

      module HowCreated
        WEBSITE = 'website'.freeze
        ADMIN_IMPORT = 'admin_import'.freeze
        ADMIN_CONTROL_PANEL = 'admin_control_panel'.freeze
        SUBMIT_CASH = 'submit_cash'.freeze
        MOBILE = 'mobile'.freeze
        PARTICIPANT_REGISTRATION = 'participant_registration'.freeze
        FUNDRAISER_REGISTRATION = 'fundraiser_registration'.freeze
        FAST_DONATION = 'fast_donation'.freeze
        RECURRING_GIFTS = 'recurring'.freeze
        GIVE_ONCE = 'give_once'.freeze
        DONATE = 'donate'.freeze
        PAYMENT_SPRING_APP = 'payment_spring_app'.freeze
      end

      module ShippingPreference
        USE_PRIMARY_ADDRESS = 'use_primary_address'.freeze
        USE_VENUE_ADDRESS = 'use_venue_address'.freeze
        USE_ALTERNATE_ADDRESS = 'use_alternate_address'.freeze
      end

      class Addresses < SBF::Client::BaseEntity
        entity_attr_accessor :primary, 'SBF::Client::ProfileAddress'
        entity_attr_accessor :secondary, 'SBF::Client::ProfileAddress'
      end

      class PhoneNumbers < SBF::Client::BaseEntity
        entity_attr_accessor :primary, 'SBF::Client::Phone'
        entity_attr_accessor :secondary, 'SBF::Client::Phone'
      end

      class EmailAddresses < SBF::Client::BaseEntity
        entity_attr_accessor :primary, 'SBF::Client::ProfileEmailAddress'
        entity_attr_accessor :secondary, 'SBF::Client::ProfileEmailAddress'
      end

      class Totals < SBF::Client::BaseEntity
        attr_reader :amount_raised
        attr_reader :donations
      end
    end

    class PartialOrganization < SBF::Client::Organization
      attr_accessor :id, :type
    end

    class FullOrganization < SBF::Client::Organization
      class ContactName < SBF::Client::BaseEntity
        attr_reader :name
        entity_attr_accessor :name_pieces, 'SBF::Client::NamePieces'
      end

      attr_accessor :id
      attr_accessor :salesforce_id
      attr_accessor :salesforce_account_id
      attr_accessor :type
      attr_accessor :status
      attr_reader :display_name
      attr_accessor :display_name_override
      attr_accessor :name
      entity_attr_accessor :contact, 'SBF::Client::FullOrganization::ContactName'
      entity_attr_accessor :addresses, 'SBF::Client::Organization::Addresses'
      entity_attr_accessor :phone_numbers, 'SBF::Client::Organization::PhoneNumbers'
      entity_attr_accessor :email_addresses, 'SBF::Client::Organization::EmailAddresses'
      entity_attr_accessor :opt_out_settings, 'SBF::Client::OptOutSettings', nil, true
      attr_accessor :selected_shipping_method
      entity_attr_accessor :alternate_shipping_address, 'SBF::Client::AlternateShippingAddress', nil, true
      attr_reader :created_at, :modified_at
      attr_accessor :modified_by
      attr_accessor :how_created
      entity_attr_reader :totals, 'SBF::Client::Organization::Totals', nil, true

      def active?
        status == SBF::Client::Organization::Status::ACTIVE
      end
    end
  end
end
