require 'stbaldricks/entities/lib/base'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/default_cacheable'

module SBF
  module Client
    class Page < SBF::Client::TopLevelEntity
      include Entities::DefaultCacheable
      actions DEFAULT_CRUD_ACTIONS

      class Content < SBF::Client::BaseEntity
        attr_reader :id
        attr_accessor :is_draft
        attr_accessor :type
        attr_accessor :title
        attr_accessor :text
        attr_accessor :sort_order
        attr_reader :date_modified

        module Type
          TEXT = 'text'.freeze
          META = 'meta'.freeze
        end
      end

      attr_reader :id
      attr_accessor :title
      attr_accessor :lookup
      attr_accessor :title_tag
      attr_accessor :locked_by_profile_id
      attr_accessor :locked_at
      attr_reader :may_edit
      entity_collection_attr_accessor :content, 'SBF::Client::Page::Content'

      def meta_content
        content.select { |x| x.type == SBF::Client::Page::Content::Type::META }
      end

      def published_content
        # Sort content so meta type is at the bottom. This preserves order for rendering on page
        content.select { |x| x.is_draft == false }.sort! do |a, b|
          next b.type <=> a.type unless b.type == a.type

          a.sort_order <=> b.sort_order
        end
      end

      def draft_content
        content.select { |x| x.is_draft == true }
      end

      def editor_content
        published_content.map do |published|
          draft_content.find { |d| published.id == d.id } || published
        end
      end

      def lock_expired?(ttl = 300)
        locked_at.nil? || Time.now >= Time.parse(locked_at) + ttl
      end

      def add_draft_sections(sections)
        sections.each do |cid, data|
          # convert symbol to integer
          cid = cid.to_s.to_i

          content = published_content.find { |c| c.id == cid }
          update_draft_content_section(content, data, cid) unless content.nil?
        end
      end

      private def update_draft_content_section(content, data, cid)
        # update or create draft version of content
        draft = draft_content.find { |d| d.id == cid }
        if draft.nil?
          self.content << SBF::Client::Page::Content.new(content.to_hash.merge(is_draft: true, title: data[:title], text: data[:text]))
        else
          draft.title = data[:title]
          draft.text = data[:text]
        end
      end
    end
  end
end
