require 'stbaldricks/endpoints/photo'
require 'stbaldricks/entities/campaign'
require 'stbaldricks/entities/event'
require 'stbaldricks/entities/event_supporter'
require 'stbaldricks/entities/fund'
require 'stbaldricks/entities/fundraiser'
require 'stbaldricks/entities/kid'
require 'stbaldricks/entities/memorial'
require 'stbaldricks/entities/participant'
require 'stbaldricks/entities/team'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/concerns/model_type_concern'

module SBF
  module Client
    class Photo < SBF::Client::TopLevelEntity
      endpoint SBF::Client::PhotoEndpoint
      actions DEFAULT_CRUD_ACTIONS

      def upload(image_data_url, with = {})
        photo = endpoint.upload(image_data_url, self, with)
        photo.errors? ? false : photo
      end

      def self.upload(*args)
        endpoint.send(:upload, *args)
      end

      attr_accessor :id
      attr_accessor :caption
      attr_accessor :sort_order
      attr_accessor :year
      attr_accessor :category
      attr_accessor :file
      attr_accessor :filename
      multitype_attr_accessor(
        :model,
        [
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::PARTICIPANT },
            'SBF::Client::Photo::FullParticipant',
            'SBF::Client::Photo::PartialParticipant'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::FUNDRAISER },
            'SBF::Client::Photo::FullFundraiser',
            'SBF::Client::Photo::PartialFundraiser'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::FUND },
            'SBF::Client::Photo::FullFund',
            'SBF::Client::Photo::PartialFund'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::CAMPAIGN },
            'SBF::Client::Photo::FullCampaign',
            'SBF::Client::Photo::PartialCampaign'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::TEAM },
            'SBF::Client::Photo::FullTeam',
            'SBF::Client::Photo::PartialTeam'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::EVENT },
            'SBF::Client::Photo::FullEvent',
            'SBF::Client::Photo::PartialEvent'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::KID },
            'SBF::Client::Photo::FullKid',
            'SBF::Client::Photo::PartialKid'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::EVENT_SUPPORTER },
            'SBF::Client::Photo::FullEventSupporter',
            'SBF::Client::Photo::PartialEventSupporter'
          ],
          [
            ->(v) { v[:model_type] == SBF::Client::Photo::ModelType::MEMORIAL },
            'SBF::Client::Photo::FullMemorial',
            'SBF::Client::Photo::PartialMemorial'
          ]
        ],
        optional: false
      )
      attr_accessor :base_photo_url
      attr_reader :original_file_id
      attr_reader :original_filename

      def to_json(*a)
        to_hash.reject { |k, _v| %i(file filename).include? k }.to_json(*a)
      end

      module Category
        SHAVEE_AFTER = 'shavee_after'.freeze
        SHAVEE_BEFORE = 'shavee_before'.freeze
        KID = 'kid'.freeze
        PARTICIPANT = 'participant'.freeze
        TEAM_LOGO = 'team_logo'.freeze
        VENUE_LOGO = 'venue_logo'.freeze
        USER_AVATAR = 'user_avatar'.freeze
        GALLERY = 'gallery'.freeze
        DEFAULT_PHOTO = 'default_photo'.freeze
        CAMPAIGN_LOGO = 'campaign_logo'.freeze
        FUND_LOGO = 'fund_logo'.freeze
        FUNDRAISER_LOGO = 'fundraiser_logo'.freeze
        EVENT_SUPPORTER_LOGO = 'event_supporter_logo'.freeze
        MEMORIAL = 'memorial'.freeze
        COVER_ORIGINAL = 'cover_original'.freeze
        COVER_DEFAULT = 'cover_default'.freeze
      end

      module ModelType
        EVENT = 'event'.freeze
        TEAM = 'team'.freeze
        PARTICIPANT = 'participant'.freeze
        FUNDRAISER = 'fundraiser'.freeze
        KID = 'kid'.freeze
        FUND = 'fund'.freeze
        CAMPAIGN = 'campaign'.freeze
        EVENT_SUPPORTER = 'event_supporter'.freeze
        MEMORIAL = 'memorial'.freeze
      end

      class FullParticipant < SBF::Client::FullParticipant
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::PARTICIPANT
        end
      end

      class PartialParticipant < SBF::Client::PartialParticipant
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::PARTICIPANT
        end
      end

      class FullFundraiser < SBF::Client::FullFundraiser
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::FUNDRAISER
        end
      end

      class PartialFundraiser < SBF::Client::PartialFundraiser
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::FUNDRAISER
        end
      end

      class FullTeam < SBF::Client::FullTeam
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::TEAM
        end
      end

      class PartialTeam < SBF::Client::PartialTeam
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::TEAM
        end
      end

      class FullEvent < SBF::Client::FullEvent
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::EVENT
        end
      end

      class PartialEvent < SBF::Client::PartialEvent
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::EVENT
        end
      end

      class FullKid < SBF::Client::FullKid
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::KID
        end
      end

      class PartialKid < SBF::Client::PartialKid
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::KID
        end
      end

      class FullFund < SBF::Client::FullFund
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::FUND
        end
      end

      class PartialFund < SBF::Client::PartialFund
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::FUND
        end
      end

      class FullCampaign < SBF::Client::FullCampaign
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::CAMPAIGN
        end
      end

      class PartialCampaign < SBF::Client::PartialCampaign
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::CAMPAIGN
        end
      end

      class FullEventSupporter < SBF::Client::FullEventSupporter
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::EVENT_SUPPORTER
        end
      end

      class PartialEventSupporter < SBF::Client::PartialEventSupporter
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::EVENT_SUPPORTER
        end
      end

      class FullMemorial < SBF::Client::FullMemorial
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::MEMORIAL
        end
      end

      class PartialMemorial < SBF::Client::PartialMemorial
        include ModelTypeConcern
        attr_reader :model_type

        def model_type
          SBF::Client::Photo::ModelType::MEMORIAL
        end
      end
    end
  end
end
