require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/endpoints/search'

module SBF
  module Client
    class Search < SBF::Client::TopLevelEntity
      endpoint SBF::Client::SearchEndpoint
      action :find

      module Type
        FUND = 'fund'.freeze
        CAMPAIGN = 'campaign'.freeze
        EVENT = 'event'.freeze
        TEAM = 'team'.freeze
        PARTICIPANT = 'participant'.freeze
        FUNDRAISER = 'fundraiser'.freeze
        KID = 'kid'.freeze
        MEMORIAL = 'memorial'.freeze
      end

      module Status
        ACTIVE = 2
      end

      module Roles
        module Type
          SHAVEE = 1
          BARBER = 2
          TREASURER = 3
          SHAVEE_RECRUITER = 4
          ORGANIZER = 5
          TEAM_CAPTAIN = 6
          LEAD_TREASURER = 8
          LEAD_ORGANIZER = 9
          BARBER_COORDINATOR = 10
          EVENT_SUPPORT_VOLUNTEER = 98
          INDIVIDUAL_FUNDRAISER = 99
        end
      end

      class Fund < SBF::Client::BaseEntity
        attr_reader :fund_id
        attr_reader :slug
        attr_reader :name
        attr_reader :website
        attr_reader :disabled
        attr_reader :event_year
        attr_reader :total_donations
        attr_reader :total_events
        attr_reader :total_participants
        attr_reader :total_shavees
        attr_reader :mypage_url
        attr_reader :default_photo_url
        attr_reader :avatar_photo_url
      end

      class Campaign < SBF::Client::BaseEntity
        attr_reader :campaign_id
        attr_reader :slug
        attr_reader :name
        attr_reader :website
        attr_reader :disabled
        attr_reader :event_year
        attr_reader :total_donations
        attr_reader :total_events
        attr_reader :total_participants
        attr_reader :total_shavees
        attr_reader :mypage_url
      end

      class Event < SBF::Client::BaseEntity
        attr_reader :venue_name
        attr_reader :event_title
        attr_reader :zip_code
        attr_reader :address1
        attr_reader :city
        attr_reader :state
        attr_reader :state_full
        attr_reader :country
        attr_reader :country_full
        attr_reader :address2
        attr_reader :event_access_type_id
        attr_reader :event_date
        attr_reader :event_year
        attr_reader :event_id
        attr_reader :is_website_allowed
        attr_reader :is_expired
        attr_reader :start_time
        attr_reader :end_time
        attr_reader :status_id
        attr_reader :website_url
        attr_reader :total_donations
        attr_reader :total_participants
        attr_reader :total_shavees
        attr_reader :icon
        attr_reader :mypage_url
        attr_reader :default_photo_url
        attr_reader :avatar_photo_url
        attr_reader :closed_roles
      end

      class Team < SBF::Client::BaseEntity
        attr_reader :team_name
        attr_reader :venue_name
        attr_reader :event_title
        attr_reader :zip_code
        attr_reader :address1
        attr_reader :city
        attr_reader :state
        attr_reader :state_full
        attr_reader :country_full
        attr_reader :country
        attr_reader :event_date
        attr_reader :event_id
        attr_reader :event_year
        attr_reader :event_status_id
        attr_reader :team_id
        attr_reader :is_website_allowed
        attr_reader :status_id
        attr_reader :total_donations
        attr_reader :total_participants
        attr_reader :total_shavees
        attr_reader :icon
        attr_reader :mypage_url
        attr_reader :default_photo_url
        attr_reader :avatar_photo_url
      end

      class Participant < SBF::Client::BaseEntity
        attr_reader :participant_id
        attr_reader :event_year
        attr_reader :raw_display_name
        attr_reader :display_name
        attr_reader :team_name
        attr_reader :venue_name
        attr_reader :event_title
        attr_reader :zip_code
        attr_reader :city
        attr_reader :state
        attr_reader :state_full
        attr_reader :participant_state
        attr_reader :participant_state_full
        attr_reader :country
        attr_reader :country_full
        attr_reader :participant_country
        attr_reader :participant_country_full
        attr_reader :event_date
        attr_reader :event_id
        attr_reader :is_website_allowed
        attr_reader :profile_id
        attr_reader :status_id
        attr_reader :event_status_id
        attr_reader :team_id
        attr_reader :virtual_participation_date
        attr_reader :total_donations
        attr_reader :icon
        attr_reader :mypage_url
        attr_reader :default_photo_url
        attr_reader :avatar_photo_url
      end

      class Fundraiser < SBF::Client::BaseEntity
        attr_reader :fundraiser_id
        attr_reader :event_year
        attr_reader :event_id
        attr_reader :event_status_id
        attr_reader :event_title
        attr_reader :team_id
        attr_reader :title
        attr_reader :status_id
        attr_reader :profile_id
        attr_reader :raw_display_name
        attr_reader :display_name
        attr_reader :start_date
        attr_reader :start_time
        attr_reader :end_date
        attr_reader :end_time
        attr_reader :zip_code
        attr_reader :venue_name
        attr_reader :team_name
        attr_reader :event_venue_name
        attr_reader :address1
        attr_reader :address2
        attr_reader :city
        attr_reader :state
        attr_reader :state_full
        attr_reader :country
        attr_reader :country_full
        attr_reader :total_donations
        attr_reader :icon
        attr_reader :mypage_url
        attr_reader :default_photo_url
        attr_reader :avatar_photo_url
      end

      class Kid < SBF::Client::BaseEntity
        attr_reader :kid_id
        attr_reader :display_name
        attr_reader :city
        attr_reader :state
        attr_reader :state_full
        attr_reader :country_full
        attr_reader :country
        attr_reader :diagnosis_slug
        attr_reader :diagnosis_other
        attr_reader :status_id
        attr_reader :treatment_status_slug
        attr_reader :treatment_status_other
        attr_reader :mypage_url
        attr_reader :default_photo_url
        attr_reader :avatar_photo_url
      end

      class Memorial < SBF::Client::BaseEntity
        attr_reader :memorial_id
        attr_reader :tribute_name
        attr_reader :display_name
        attr_reader :status_id
        attr_reader :city
        attr_reader :state
        attr_reader :state_full
        attr_reader :country_full
        attr_reader :country
        attr_reader :tribute_birthday
        attr_reader :tribute_date_of_passing
        attr_reader :expiration_date
        attr_reader :total_donations
        attr_reader :mypage_url
        attr_reader :default_photo_url
        attr_reader :avatar_photo_url
      end
    end
  end
end
