require 'stbaldricks/endpoints/shave_schedule'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/permissions'

module SBF
  module Client
    class ShaveSchedule < SBF::Client::TopLevelEntity
      endpoint SBF::Client::ShaveScheduleEndpoint
      actions DEFAULT_CRUD_ACTIONS
      action :notify

      def update(with = {})
        endpoint.update(id, year, self, with)
      end

      class TimeSelectionPermissions < SBF::Client::BaseEntity
        attr_accessor :shavees, :team_captains

        def shavees?
          shavees
        end

        def team_captains?
          team_captains
        end
      end

      class ShaveScheduleAssignment < SBF::Client::BaseEntity
        module Type
          RESERVED = 'reserved'.freeze
          SHAVEE = 'shavee'.freeze
        end

        attr_accessor :id
        attr_accessor :type
        attr_accessor :participant
        attr_accessor :is_notification_sent
        attr_accessor :shave_time
      end

      attr_accessor :id
      attr_accessor :year
      attr_accessor :start_time
      attr_accessor :end_time
      attr_accessor :shave_duration_in_minutes
      attr_accessor :number_of_seats
      attr_accessor :is_published
      entity_attr_accessor :time_selection_permissions, 'SBF::Client::ShaveSchedule::TimeSelectionPermissions'
      entity_collection_attr_accessor :assignments, 'SBF::Client::ShaveSchedule::ShaveScheduleAssignment'
      entity_attr_reader :permissions, 'SBF::Client::Permissions', nil, true

      def published?
        is_published
      end

      def shavees_can_schedule_times?
        time_selection_permissions.shavees?
      end

      def team_captains_can_schedule_times?
        time_selection_permissions.team_captains?
      end

      def owner?
        @permissions.nil? ? false : @permissions.owner?
      end
    end
  end
end
