require 'stbaldricks/entities/participant'
require 'stbaldricks/entities/fund'
require 'stbaldricks/entities/event'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/fundraising_page'
require 'stbaldricks/entities/lib/permissions'
require 'stbaldricks/entities/lib/default_cacheable'
require 'stbaldricks/entities/concerns/donation_recipient_concern'
require 'stbaldricks/entities/concerns/event_year_concern'
require 'stbaldricks/entities/concerns/fundraising_page_concern'

module SBF
  module Client
    class Team < SBF::Client::TopLevelEntity
      include FundraisingPageConcern, EventYearConcern, DonationRecipientConcern, Entities::DefaultCacheable
      actions DEFAULT_CRUD_ACTIONS

      blacklist_action :create

      disallow_instantiation

      module Status
        ACTIVE = 'active'.freeze
        PENDING = 'pending'.freeze
        SUSPENDED = 'suspended'.freeze
      end

      module HowCreated
        WEBSITE = 'website'.freeze
        ADMIN_IMPORT = 'admin_import'.freeze
        ADMIN_CONTROL_PANEL = 'admin_control_panel'.freeze
        MOBILE = 'mobile'.freeze
        PARTICIPANT_IMPORT = 'participant_import'.freeze
      end

      module FundRelationshipType
        NO_ASSOCIATION = 'no_association'.freeze
        INDIRECT_ONE = 'indirect_one'.freeze
        INDIRECT_MULTIPLE = 'indirect_multiple'.freeze
        DIRECT = 'direct'.freeze
        DIRECT_BUT_LOCKED = 'direct_but_locked'.freeze
        INHERITED_FROM_EVENT = 'inherited_from_event'.freeze
      end

      class Photos < SBF::Client::BaseEntity
        attr_reader :avatar, :default, :icon
      end

      class Totals < SBF::Client::BaseEntity
        attr_reader :fundraisers, :participants, :shavees, :amount_raised
      end

      class Rankings < SBF::Client::BaseEntity
        class Ranking < SBF::Client::BaseEntity
          attr_reader :rank, :total
        end

        entity_attr_reader :event, 'SBF::Client::Team::Rankings::Ranking'
      end
    end

    class PartialTeam < Team
      attr_accessor :id
    end

    class FullTeam < Team
      extend Forwardable
      def_delegator :@photos, :avatar, :avatar_photo_url

      attr_accessor :id
      attr_accessor :name
      attr_accessor :status
      attr_reader :display_name
      entity_attr_accessor :event, 'SBF::Client::FullEvent', 'SBF::Client::PartialEvent', true
      entity_attr_accessor :fund, 'SBF::Client::FullFund', 'SBF::Client::PartialFund', true
      attr_accessor :fund_relationship_type
      attr_accessor :offline_donation_uuid, :offline_donation_form
      attr_accessor :fundraising_goal, :fundraising_message
      entity_attr_accessor :fundraising_page, 'SBF::Client::FundraisingPage', nil, true
      entity_attr_reader :photos, 'SBF::Client::Team::Photos'
      entity_attr_accessor :captain, 'SBF::Client::FullParticipant'
      entity_attr_reader :rankings, 'SBF::Client::Team::Rankings', nil, true
      entity_attr_reader :totals, 'SBF::Client::Team::Totals'
      attr_reader :created_at, :modified_at
      attr_accessor :modified_by
      attr_accessor :how_created
      entity_attr_reader :permissions, 'SBF::Client::Permissions', nil, true
      entity_attr_accessor :third_party_media, 'SBF::Client::ThirdPartyMedia', nil, true

      def active?
        status == SBF::Client::Team::Status::ACTIVE
      end

      def pending?
        status == SBF::Client::Team::Status::PENDING
      end

      def suspended?
        status == SBF::Client::Team::Status::SUSPENDED
      end
    end
  end
end
