require 'stbaldricks/endpoints/user'
require 'stbaldricks/entities/person'
require 'stbaldricks/entities/organization'
require 'stbaldricks/entities/lib/top_level'
require 'stbaldricks/entities/lib/cacheable'

module SBF
  module Client
    class User < SBF::Client::TopLevelEntity
      include Entities::Cacheable

      endpoint SBF::Client::UserEndpoint
      actions DEFAULT_CRUD_ACTIONS
      action :login!
      action :list_entities_user_has_permission
      action :change_password
      action :disconnect_facebook

      cache_id -> { [person.id, type] }
      cache_id_from_hash ->(h) { [h[:person][:id], h[:type]] }

      module AccessScope
        ADMIN = 'admin'.freeze
        MEMBER = 'member'.freeze
      end

      module ProviderType
        SBF = 'sbf'.freeze
        FACEBOOK = 'facebook'.freeze
        TWITTER = 'twitter'.freeze

        # rubocop:disable Style/MutableConstant
        IDS = {
          SBF => 1,
          FACEBOOK => 2,
          TWITTER => 3
        }
        # rubocop:enable Style/MutableConstant
        IDS.default = 0
        IDS.freeze

        def self.to_s(type_id)
          IDS.key(type_id)
        end

        def self.value_of(type)
          IDS[type]
        end
      end

      attr_accessor :type, :access_scope, :username, :password, :token, :identifier

      multitype_attr_accessor(
        :person,
        [
          [
            ->(v) { v[:type] == SBF::Client::Organization::Type::ORGANIZATION_FOUNDATION },
            'SBF::Client::FullOrganization',
            'SBF::Client::PartialOrganization'
          ],
          [
            ->(v) { v[:type] == SBF::Client::Organization::Type::BUSINESS },
            'SBF::Client::FullOrganization',
            'SBF::Client::PartialOrganization'
          ],
          [
            ELSE,
            'SBF::Client::FullPerson',
            'SBF::Client::PartialPerson'
          ]
        ]
      )

      attr_reader :created_at, :modified_at, :last_login_at
    end
  end
end
