module SBF
  module Client
    module Permission
      PUBLIC_ACCESS = 3
      ADMIN = 1
      MEMBERS = 2
      REGISTRATION = 4

      MANAGE_USERS = 101
      ADD_USERS = 102
      DELETE_USERS = 103
      EDIT_USERS = 104

      MANAGE_GROUPS = 108
      ADD_GROUPS = 105
      DELETE_GROUPS = 106
      EDIT_GROUPS = 107

      MANAGE_SYSTEM = 109

      # Event permissions
      EVENT_MANAGE = 200
      EVENT_MANAGE_INFO = 201
      EVENT_MANAGE_ACTIVITIES = 202
      EVENT_ADD_ACTIVITIES = 203
      EVENT_DELETE_ACTIVITIES = 204
      EVENT_MANAGE_PHOTOS = 205
      EVENT_ADD_PHOTOS = 206
      EVENT_DELETE_PHOTOS = 207
      EVENT_MANAGE_SUPPORTERS = 208
      EVENT_ADD_SUPPORTERS = 209
      EVENT_DELETE_SUPPORTERS = 210
      EVENT_MANAGE_PARTICIPANTS = 211
      EVENT_MANAGE_INDIVIDUAL_FUNDRAISERS = 212
      EVENT_MANAGE_KIDS = 213
      EVENT_ADD_KID = 214
      EVENT_DELETE_KID = 215
      EVENT_VIEW_DONATIONS = 216
      EVENT_CASH_DONATIONS = 217
      EVENT_MATERIALS = 218
      EVENT_MASS_PASSWORD_RESET = 219
      EVENT_FAST_REGISTRATION = 220
      EVENT_DELETE_EVENT = 221
      EVENT_VIEW_EVENT_PERFORMANCE = 222
      EVENT_MANAGE_EVENT_SUPPORT_VOLUNTEERS = 223

      # text descriptions found in LookupUtility
      EVENT_MEMBER_UPDATE_EVENT_INFORMATION = 300
      EVENT_MEMBER_ACTIVITIES = 301
      EVENT_MEMBER_PHOTO_GALLERY = 302
      EVENT_MEMBER_SUPPORTERS = 303
      EVENT_MEMBER_MANAGE_INDIVIDUAL_FUNDRAISERS_AND_ORGANIZERS = 305
      EVENT_MEMBER_MANAGE_HONORED_KIDS_WITH_CANCER = 306
      EVENT_MEMBER_VIEW_FUNDRAISING_REPORTS = 307
      EVENT_MEMBER_DOWNLOAD_EVENT_PLANNING_MATERIALS = 308
      EVENT_MEMBER_VIEW_PARTICIPANT_CONTACT_LIST = 309
      EVENT_MEMBER_MANAGE_PARTICIPANT_ROLES = 310
      EVENT_MEMBER_MANAGE_PARTICIPANT_PHOTOS = 311
      EVENT_MEMBER_MANAGE_PARTICIPANT_PERMISSIONS = 312
      EVENT_MEMBER_SUBMIT_CASH_DONATIONS = 313
      EVENT_MEMBER_VIEW_TEAM_TOTALS_LIST = 314
      EVENT_MEMBER_COMMUNICATE = 315
      EVENT_MEMBER_FAST_DONATION = 316
      EVENT_MEMBER_MANAGE_EVENT_SUPPORT_VOLUNTEERS_AND_ORGANIZERS = 317

      # Participant Role Permissions
      EVENT_ORGANIZER = 1905
      EVENT_BARBER_COORDINATOR = 1911
      EVENT_SHAVEE_RECRUITER = 1913

      MOVE_PARTICIPANT_TO_NEW_PROFILE = 1912

      COACH_MANAGE = 401

      DONATION_VIEW = 501

      DONATION_EDIT = 502

      FIRESPRING_EMPLOYEE = 666

      EVENT_SUPPLY_ORDER_MANAGE = 701

      VIEW_EVENT_SUPPLY_ORDERS = 702

      KID_MANAGE = 801

      PARTICIPANT_MANAGE = 901

      PROFILE_MANAGE = 1001

      TEAM_MANAGE = 1101

      DEDUPLICATOR_MANAGE = 1201

      DOCUMENT_LIBRARY_MANAGE = 1501

      MESSAGES_MANAGE = 1701

      MANAGE_MEDIA_MANAGER = 1801
      CONTENT_MANAGE = 2203

      TRIGGER_MANAGE = 2001
      TRIGGER_DELETE = 2002
      TRIGGER_EDIT = 2003
      TRIGGER_ADD = 2004

      INTERNATIONAL_PARTNER_MANAGE = 2301

      GRANTS_MANAGE = 2601
      GRANTS_VIEW = 2602

      CAMPAIGNS_MANAGE = 2701

      FUNDS_MANAGE = 2801

      FUNDRAISER_MANAGE = 2901
      FUNDRAISER_DELETE_FUNDRAISER = 2902

      EVENT_APPLICATION_MANAGE = 2202

      # Team Permissions
      TEAM_TEAM_CAPTAIN = 2101

      # Memorial Permissions
      MEMORIAL_VIEW = 3001
      MEMORIAL_MANAGE = 3002

      # Challenge Permissions
      CHALLENGE_VIEW = 3101
      CHALLENGE_MANAGE = 3102

      @list = {
        PUBLIC_ACCESS => :PUBLIC_ACCESS,
        ADMIN => :ADMIN,
        MEMBERS => :MEMBERS,
        REGISTRATION => :REGISTRATION,
        MANAGE_USERS => :MANAGE_USERS,
        ADD_USERS => :ADD_USERS,
        DELETE_USERS => :DELETE_USERS,
        EDIT_USERS => :EDIT_USERS,
        MANAGE_GROUPS => :MANAGE_GROUPS,
        ADD_GROUPS => :ADD_GROUPS,
        DELETE_GROUPS => :DELETE_GROUPS,
        EDIT_GROUPS => :EDIT_GROUPS,
        MANAGE_SYSTEM => :MANAGE_SYSTEM,
        # Event permissions
        EVENT_MANAGE => :EVENT_MANAGE,
        EVENT_MANAGE_INFO => :EVENT_MANAGE_INFO,
        EVENT_MANAGE_ACTIVITIES => :EVENT_MANAGE_ACTIVITIES,
        EVENT_ADD_ACTIVITIES => :EVENT_ADD_ACTIVITIES,
        EVENT_DELETE_ACTIVITIES => :EVENT_DELETE_ACTIVITIES,
        EVENT_MANAGE_PHOTOS => :EVENT_MANAGE_PHOTOS,
        EVENT_ADD_PHOTOS => :EVENT_ADD_PHOTOS,
        EVENT_DELETE_PHOTOS => :EVENT_DELETE_PHOTOS,
        EVENT_MANAGE_SUPPORTERS => :EVENT_MANAGE_SUPPORTERS,
        EVENT_ADD_SUPPORTERS => :EVENT_ADD_SUPPORTERS,
        EVENT_DELETE_SUPPORTERS => :EVENT_DELETE_SUPPORTERS,
        EVENT_MANAGE_PARTICIPANTS => :EVENT_MANAGE_PARTICIPANTS,
        EVENT_MANAGE_INDIVIDUAL_FUNDRAISERS => :EVENT_MANAGE_INDIVIDUAL_FUNDRAISERS,
        EVENT_MANAGE_EVENT_SUPPORT_VOLUNTEERS => :EVENT_MANAGE_EVENT_SUPPORT_VOLUNTEERS,
        EVENT_MANAGE_KIDS => :EVENT_MANAGE_KIDS,
        EVENT_ADD_KID => :EVENT_ADD_KID,
        EVENT_DELETE_KID => :EVENT_DELETE_KID,
        EVENT_VIEW_DONATIONS => :EVENT_VIEW_DONATIONS,
        EVENT_CASH_DONATIONS => :EVENT_CASH_DONATIONS,
        EVENT_MATERIALS => :EVENT_MATERIALS,
        EVENT_MASS_PASSWORD_RESET => :EVENT_MASS_PASSWORD_RESET,
        EVENT_FAST_REGISTRATION => :EVENT_FAST_REGISTRATION,
        EVENT_DELETE_EVENT => :EVENT_DELETE_EVENT,
        EVENT_MEMBER_UPDATE_EVENT_INFORMATION => :EVENT_MEMBER_UPDATE_EVENT_INFORMATION,
        EVENT_MEMBER_ACTIVITIES => :EVENT_MEMBER_ACTIVITIES,
        EVENT_MEMBER_PHOTO_GALLERY => :EVENT_MEMBER_PHOTO_GALLERY,
        EVENT_MEMBER_SUPPORTERS => :EVENT_MEMBER_SUPPORTERS,
        EVENT_MEMBER_MANAGE_INDIVIDUAL_FUNDRAISERS_AND_ORGANIZERS => :EVENT_MEMBER_MANAGE_INDIVIDUAL_FUNDRAISERS_AND_ORGANIZERS,
        EVENT_MEMBER_MANAGE_EVENT_SUPPORT_VOLUNTEERS_AND_ORGANIZERS => :EVENT_MEMBER_MANAGE_EVENT_SUPPORT_VOLUNTEERS_AND_ORGANIZERS,
        EVENT_MEMBER_MANAGE_HONORED_KIDS_WITH_CANCER => :EVENT_MEMBER_MANAGE_HONORED_KIDS_WITH_CANCER,
        EVENT_MEMBER_VIEW_FUNDRAISING_REPORTS => :EVENT_MEMBER_VIEW_FUNDRAISING_REPORTS,
        EVENT_MEMBER_DOWNLOAD_EVENT_PLANNING_MATERIALS => :EVENT_MEMBER_DOWNLOAD_EVENT_PLANNING_MATERIALS,
        EVENT_MEMBER_VIEW_PARTICIPANT_CONTACT_LIST => :EVENT_MEMBER_VIEW_PARTICIPANT_CONTACT_LIST,
        EVENT_MEMBER_MANAGE_PARTICIPANT_ROLES => :EVENT_MEMBER_MANAGE_PARTICIPANT_ROLES,
        EVENT_MEMBER_MANAGE_PARTICIPANT_PHOTOS => :EVENT_MEMBER_MANAGE_PARTICIPANT_PHOTOS,
        EVENT_MEMBER_MANAGE_PARTICIPANT_PERMISSIONS => :EVENT_MEMBER_MANAGE_PARTICIPANT_PERMISSIONS,
        EVENT_MEMBER_SUBMIT_CASH_DONATIONS => :EVENT_MEMBER_SUBMIT_CASH_DONATIONS,
        EVENT_MEMBER_VIEW_TEAM_TOTALS_LIST => :EVENT_MEMBER_VIEW_TEAM_TOTALS_LIST,
        EVENT_MEMBER_COMMUNICATE => :EVENT_MEMBER_COMMUNICATE,
        EVENT_MEMBER_FAST_DONATION => :EVENT_MEMBER_FAST_DONATION,
        # Participant Role Permissions
        EVENT_ORGANIZER => :EVENT_ORGANIZER,
        EVENT_BARBER_COORDINATOR => :EVENT_BARBER_COORDINATOR,
        EVENT_SHAVEE_RECRUITER => :EVENT_SHAVEE_RECRUITER,
        MOVE_PARTICIPANT_TO_NEW_PROFILE => :MOVE_PARTICIPANT_TO_NEW_PROFILE,
        COACH_MANAGE => :COACH_MANAGE,
        DONATION_VIEW => :DONATION_VIEW,
        DONATION_EDIT => :DONATION_EDIT,
        FIRESPRING_EMPLOYEE => :FIRESPRING_EMPLOYEE,
        EVENT_SUPPLY_ORDER_MANAGE => :EVENT_SUPPLY_ORDER_MANAGE,
        VIEW_EVENT_SUPPLY_ORDERS => :VIEW_EVENT_SUPPLY_ORDERS,
        KID_MANAGE => :KID_MANAGE,
        PARTICIPANT_MANAGE => :PARTICIPANT_MANAGE,
        PROFILE_MANAGE => :PROFILE_MANAGE,
        TEAM_MANAGE => :TEAM_MANAGE,
        DEDUPLICATOR_MANAGE => :DEDUPLICATOR_MANAGE,
        DOCUMENT_LIBRARY_MANAGE => :DOCUMENT_LIBRARY_MANAGE,
        MESSAGES_MANAGE => :MESSAGES_MANAGE,
        MANAGE_MEDIA_MANAGER => :MANAGE_MEDIA_MANAGER,
        CONTENT_MANAGE => :CONTENT_MANAGE,
        TRIGGER_MANAGE => :TRIGGER_MANAGE,
        TRIGGER_DELETE => :TRIGGER_DELETE,
        TRIGGER_EDIT => :TRIGGER_EDIT,
        TRIGGER_ADD => :TRIGGER_ADD,
        INTERNATIONAL_PARTNER_MANAGE => :INTERNATIONAL_PARTNER_MANAGE,
        GRANTS_MANAGE => :GRANTS_MANAGE,
        GRANTS_VIEW => :GRANTS_VIEW,
        CAMPAIGNS_MANAGE => :CAMPAIGNS_MANAGE,
        FUNDS_MANAGE => :FUNDS_MANAGE,
        FUNDRAISER_MANAGE => :FUNDRAISER_MANAGE,
        FUNDRAISER_DELETE_FUNDRAISER => :FUNDRAISER_DELETE_FUNDRAISER,
        EVENT_APPLICATION_MANAGE => :EVENT_APPLICATION_MANAGE,
        # Team Permissions
        TEAM_TEAM_CAPTAIN => :TEAM_TEAM_CAPTAIN,
        # Memorial Permissions
        MEMORIAL_VIEW => :MEMORIAL_VIEW,
        MEMORIAL_MANAGE => :MEMORIAL_MANAGE,
        # Challenge Permissions
        CHALLENGE_VIEW => :CHALLENGE_VIEW,
        CHALLENGE_MANAGE => :CHALLENGE_MANAGE
      }.freeze

      class << self
        attr_reader :list
      end
    end
  end
end
