require 'active_support/core_ext/object/blank'

class String
  SNAKE_CASE_REGEX = Regexp.new('(?<!^)([[:upper:]])|([[:upper:]])')
  ID_REGEX = Regexp.new('(^|_)i_d')
  UUID_REGEX = Regexp.new('(^|_)u_u_id')
  URL_REGEX = Regexp.new('(^|_)u_r_l')
  MODULE_SEPARATOR = '::'.freeze

  def camel_case
    x = split('_').map do |e|
      case e
      when 'id', 'uuid', 'url'
        e.upcase

      else
        # capitalize first character (capitalize will lowercase the rest of the word)
        e[0] = e[0].capitalize
        e

      end
    end

    x.join
  end

  def snake_case
    gsub(SNAKE_CASE_REGEX) do
      (Regexp.last_match[1] ? "_#{Regexp.last_match[1]}" : Regexp.last_match[2]).downcase
    end.sub(ID_REGEX, '_id').sub(UUID_REGEX, '_uuid').sub(URL_REGEX, '_url')
  end

  def blank?
    strip.empty?
  end

  def deconstantize
    rpartition(MODULE_SEPARATOR).first
  end

  def demodulize
    rpartition(MODULE_SEPARATOR).last
  end

  def to_class
    split(MODULE_SEPARATOR).reduce(Object) { |a, e| a.const_get(e.to_sym) }
  rescue
    nil
  end

  def to_b
    return false if self == 'false' || self == '0'
    return true if self == 'true' || self == '1'

    nil
  end
end
